/*
  Rhino test framework

   Copyright (C) 2011, 2012  Red Hat

This file is part of IcedTea.

IcedTea is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

IcedTea is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with IcedTea; see the file COPYING.  If not, write to the
Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
02110-1301 USA.

Linking this library statically or dynamically with other modules is
making a combined work based on this library.  Thus, the terms and
conditions of the GNU General Public License cover the whole
combination.

As a special exception, the copyright holders of this library give you
permission to link this library with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on this library.  If you modify this library, you may extend
this exception to your version of the library, but you are not
obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
 */

package org.RhinoTests.Reporter;

import java.io.File;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

/**
 * Log pages generator.
 *
 * @author Pavel Tisnovsky
 */
public class LogPagesGenerator
{

    public static void generate(Map<String, Map<String, List<String>>> testResults, CommandLineParameters params)
    {
        for (Entry<String, Map<String, List<String>>> testResult : testResults.entrySet()) {
            createOneLogPage(testResult.getKey(), testResult.getValue(), params);
        }
    }

    private static void createOneLogPage(String testDate, Map<String, List<String>> testResult, CommandLineParameters params)
    {
        List<String> summaryLines = prepareSummaryLinesForLogPage(params);
        List<String> resultLines = prepareResultLinesForLogPage(testResult);
        List<String> template = FileUtils.readTextFile(params.getTemplateDir() + "/log.html");
        List<String> out = generateOutputForOneLogPage(testDate, summaryLines, resultLines, template);
        // write list of string to a file with given name
        File outputFile = new File(params.getReportDir(), "log_" + testDate + ".html");
        FileUtils.writeTextFile(outputFile, out);
    }

    /**
     * @param testDate
     * @param summaryLines
     * @param resultLines
     * @param template
     * @return
     */
    private static List<String> generateOutputForOneLogPage(String testDate, List<String> summaryLines,
                    List<String> resultLines, List<String> template)
    {
        List<String> out = new LinkedList<String>();
        // iterate through whole template
        for (String templateLine : template) {
            // replace text in template where needed
            if (templateLine.contains("${DATE}")) {
                templateLine = templateLine.replace("${DATE}", testDate);
                out.add(templateLine);
            }
            else if (templateLine.contains("${SUMMARY}")) {
                out.addAll(summaryLines);
            }
            else if (templateLine.contains("${RESULTS}")) {
                out.addAll(resultLines);
            }
            else {
                out.add(templateLine);
            }
        }
        return out;
    }

    /**
     * @return
     */
    private static List<String> prepareSummaryLinesForLogPage(CommandLineParameters params)
    {
        List<String> summaryLines = new LinkedList<String>();
        for (String testName : params.getTests()) {
            summaryLines.add("<tr><td><a href='#" + testName + "'>" + testName + "</a></td></tr>");
        }
        return summaryLines;
    }

    /**
     * @param testResult
     * @return
     */
    private static List<String> prepareResultLinesForLogPage(Map<String, List<String>> testResult)
    {
        List<String> resultLines = new LinkedList<String>();
        for (Entry<String, List<String>> testSuite : testResult.entrySet()) {
            String testSuiteName = getTestSuiteName(testSuite);
            resultLines.add("<tr><td class='table-header' colspan='2'><a name='" + testSuiteName + "'></a>" + testSuiteName + "</td></tr>");
            boolean failed = false;
            for (String testLine : testSuite.getValue()) {
                if (testLine.startsWith("PASSED: ")) {
                    if (failed) {
                        resultLines.add("</pre></td><td>&nbsp;</td></tr>");
                        failed = false;
                    }
                    String testName = testLine.substring("PASSED: ".length());
                    resultLines.add("<tr><td>" + testName + "</td><td class='passed-text'>PASSED</td></tr>");
                }
                else if (testLine.startsWith("FAILED: ")) {
                    String testName = testLine.substring("FAILED: ".length());
                    resultLines.add("<tr><td class='failed-header'>" + testName + "</td><td class='failed-text'>FAILED</td></tr>");
                    resultLines.add("<tr><td><pre class='stack-trace'>");
                    failed = true;
                }
                else if (testLine.startsWith("SUMMARY: ")) {
                    if (failed) {
                        resultLines.add("</pre></td><td>&nbsp;</td></tr>");
                        failed = false;
                    }
                    resultLines.add("<tr><td class='error-text' colspan='2'>" + testLine + "</td></tr>");
                }
                else if (failed) {
                    resultLines.add(testLine);
                }
            }
            resultLines.add("<td><td colspan='2'>&nbsp;</td></tr>");
        }
        return resultLines;
    }

    /**
     * @param testSuite
     * @return
     */
    private static String getTestSuiteName(Entry<String, List<String>> testSuite)
    {
        String testSuiteName = testSuite.getKey();
        testSuiteName = testSuiteName.substring(testSuiteName.lastIndexOf('/') + 1, testSuiteName.lastIndexOf('.'));
        return testSuiteName;
    }

}
